"use strict";

var _layouter = _interopRequireDefault(require("../src/layouter.js"));

var _imagesMock = require("./images-mock.js");

var _deepFreeze = _interopRequireDefault(require("deep-freeze"));

var _chai = require("chai");

var _expectedOffsets = _interopRequireDefault(require("./expectedOffsets"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

/* eslint-disable */
var getItems = function getItems(count) {
  return (0, _deepFreeze["default"])(_imagesMock.testImages.slice(0, count));
};

var fs = require('fs');

var logToFile = function logToFile(text, overwrite) {
  // Define file name.
  var filename = 'test/expectedOffsets.js'; // Define log text.

  var logText = text + '\r\n'; // Save log to file.

  if (overwrite) {
    fs.writeFileSync(filename, logText, 'utf8');
  } else {
    fs.appendFileSync(filename, logText, 'utf8');
  }
};

describe('Expected Layouts', function () {
  var gallery;
  var layouter;
  var container = {};
  var styleParams = {};
  var styleParamsOptions = {};

  var getLayout = function getLayout(layoutParams) {
    return layouter.createLayout(layoutParams);
  };

  beforeEach(function () {
    var items = getItems();
    styleParamsOptions = {
      oneRow: [true, false],
      isVertical: [true, false],
      gallerySize: [100, 500],
      groupSize: [1, 3],
      groupTypes: ['1,2h,2v,3t,3b,3l,3r,3v,3h', '2h,2v,3t,3v,3h'],
      cubeImages: [true, false],
      cubeRatio: [1, 2],
      smartCrop: [true, false],
      chooseBestGroup: [true, false],
      collageDensity: [0, 1],
      minItemSize: [20, 200],
      imageMargin: [0, 100],
      fixedColumns: [0, 5]
    };
    container = {
      width: 1000,
      bounds: {
        visibleTop: 0,
        visibleBottom: 1000,
        renderedTop: 0,
        renderedBottom: 10000
      }
    };
    layouter = new _layouter["default"]({
      items: items,
      container: container,
      styleParams: styleParams
    });
  });
  it('should remain as expected', function () {
    if (process.env.IN_WALLABY) {
      return; //do not run this test in wallaby (takes too long)
    }

    var isDebug = false;
    var items = getItems(20);
    var totalOptions = [];

    for (var _i = 0, _Object$values = Object.values(styleParamsOptions); _i < _Object$values.length; _i++) {
      var option = _Object$values[_i];
      totalOptions.push((totalOptions[totalOptions.length - 1] || 1) * option.length);
    }

    var selectStyleParams = function selectStyleParams(seed) {
      var styleParams = {};
      var styleParamsArr = Object.entries(styleParamsOptions); // console.log(styleParamsArr);

      var idx = 0;

      for (var _i2 = 0, _styleParamsArr = styleParamsArr; _i2 < _styleParamsArr.length; _i2++) {
        var _styleParamsArr$_i = _styleParamsArr[_i2],
            key = _styleParamsArr$_i[0],
            value = _styleParamsArr$_i[1];
        styleParams[key] = value[Math.floor(seed / totalOptions[idx]) % value.length];
        idx++;
      }

      return styleParams;
    };

    var startTime = Date.now();

    if (isDebug) {
      logToFile('\/* eslint-disable *\/', true);
      logToFile('export default [');
    }

    for (var seed = 1; seed < totalOptions[totalOptions.length - 1]; seed++) {
      var _styleParams = selectStyleParams(seed);

      gallery = getLayout({
        items: items,
        container: container,
        styleParams: _styleParams
      });
      var offsets = gallery.items.reduce(function (str, item) {
        var os = item.offset;
        var itemStr = "{t:" + Math.round(os.top) + ",l:" + Math.round(os.left) + ",r:" + Math.round(os.right) + ",b:" + Math.round(os.bottom) + "}";
        return str + itemStr;
      }, '');

      if (isDebug) {
        if (!(seed % 100)) {
          console.log("Completed " + Math.round(100 * seed / totalOptions[totalOptions.length - 1]) + "% " + seed + "/" + totalOptions[totalOptions.length - 1] + " layouts (" + Math.floor((Date.now() - startTime) / 1000) + "s)");
        }

        logToFile('\'' + offsets + '\',');
      } else {
        if (offsets !== _expectedOffsets["default"][seed - 1]) {
          console.error("Broken SP: " + seed + ": " + Object.entries(selectStyleParams(seed)).map(function (e) {
            return e.join('=');
          }).join('|'));
        }

        (0, _chai.expect)(seed + '|' + offsets).to.equal(seed + '|' + _expectedOffsets["default"][seed - 1]);
      }
    }

    if (isDebug) {
      logToFile('];');
    }
  });
});